/**
 * DashbotParser.js
 * 
 * $Id$
 * 
 * @package		Dashbot
 * @version		0.2.0
 * @author		Masayuki Iwai <miyabi@mybdesign.com>
 * @copyright	Copyright &copy; 2010 Masayuki Iwai all rights reserved.
 **/


window.DashbotParser = function(owner)
{
	var self = this;
	
	var config = {
		"available_keys": {"name":1, "screen_name":1, "location":1, "description":1, "friends_count":1, "followers_count":1, "statuses_count":1}, 
		"available_weekdays": {"sun":1, "mon":1, "tue":1, "wed":1, "thu":1, "fri":1, "sat":1}, 
		"": ""
	};

	self.owner = owner;
	
	self.lines = [];
	self.readingIndex = 0;
	self.lastReadLine = "";
	self.regexpConditionPattern = /^\?\/(.*?[^\\](\\\\)*)\/(i)?(\s*(.+))?$/;
	self.userConditionPattern = /^\?\((\w+)\s*(==|!=|<=?|>=?|=~)\s*(\/(.*?[^\\](\\\\)*)\/(i)?|(.+?))\)(\s*(.+))?$/;
	self.dateConditionPattern = /^\?\[([^\]]*?)\s*(\(([A-Za-z,\s]+?)\))?\](\s*(.+))?$/;
	self.splitDateAndTimePattern = /^((.+)\s+)?(.+)$/;
	self.datePattern = /((\d{4}|\*)\s*\/\s*)?(\d{1,2}|\*)\s*\/\s*(\d{1,2}|\*)/;
	self.timePattern = /(\d{1,2}|\*)(\s*:\s*(\d{1,2}|\*))?/;
	
	
	//////// Methods
	self.parse = function(source, skipTweets)
	{
		self.lines = source.split("\n");
		self.readingIndex = 0;
		
		return self.parseElement(false, skipTweets);
	};
	
	
	self.readLine = function()
	{
		if(self.readingIndex < self.lines.length)
		{
			self.lastReadLine = self.lines[self.readingIndex++];
			return self.lastReadLine;
		}
		else
		{
			return null;
		}
	};
	
	
	self.parseElement = function(block, skipTweets)
	{
		var children = [], else_index = -1;
		var line, matched;
		while((line = self.readLine()) != null)
		{
			line = self.trim(line);
			
			if(line == "")
			{
				if(block)
				{
					// End of block
					break;
				}
				else
				{
					// Skip
					continue;
				}
			}
			
			if(line.substr(0, 2) == "//")
			{
				// Comment
				continue;
			}
			
			if(line.substr(0, 1) != "?")
			{
				// Tweet
				if(!skipTweets)
				{
					children.push(line);
					if(else_index < 0)
						else_index = children.length - 1;
				}
				
				continue;
			}
			
			else_index = -1;
			
			if(matched = line.match(self.regexpConditionPattern))
			{
				// Regular expression condition
				var condition = {
					"type": "regexp",
					"pattern": (!self.isUndefined(matched[1])? matched[1]: ""), 
					"options": (!self.isUndefined(matched[3])? matched[3]: ""), 
					"content": {}
				};
				
				if(!self.isUndefined(matched[4]))
				{
					condition.content = (!self.isUndefined(matched[5])? matched[5]: "");
				}
				else
				{
					if(!(condition.content = self.parseElement(true)))
						return false;
				}
				
				children.push(condition);
			}
			else if(matched = line.match(self.userConditionPattern))
			{
				// User condition
				var condition = {
					"type": "user",
					"key": (!self.isUndefined(matched[1])? matched[1]: ""), 
					"operator": (!self.isUndefined(matched[2])? matched[2]: ""), 
					"regexp_pattern": (!self.isUndefined(matched[4])? matched[4]: ""), 
					"regexp_options": (!self.isUndefined(matched[6])? matched[6]: ""), 
					"value": (!self.isUndefined(matched[7])? matched[7]: ""), 
					"content": {}
				};
				
				if(!config.available_keys[condition.key])
				{
					return false;
				}
				
				if(!self.isUndefined(matched[8]))
				{
					condition.content = (!self.isUndefined(matched[9])? matched[9]: "");
				}
				else
				{
					if(!(condition.content = self.parseElement(true)))
						return false;
				}
				
				children.push(condition);
			}
			else if(matched = line.match(self.dateConditionPattern))
			{
				// Date condition
				var spans = (!self.isUndefined(matched[1])? matched[1].split(","): ["*"]);
				var weekdays = (!self.isUndefined(matched[2])? ((!self.isUndefined(matched[3])? matched[3].split(","): null)): null);
				var content = (!self.isUndefined(matched[4])? ((!self.isUndefined(matched[5])? matched[5]: "")): null);
				
				var condition = {
					"type": "date", 
					"spans": [], 
					"weekdays": [], 
					"content": {}
				};
				
				for(var i=0; i<spans.length; i++)
				{
					var span = [], year = "", month = "", day = "", hour = "", minute = "";
					var from_to = (self.trim(spans[i])).split("-");
					for(var j=0; j<from_to.length; j++)
					{
						if(from_to[j] == "")
						{
							span.push("*");
							continue;
						}
						
						if(!(matched = from_to[j].match(self.splitDateAndTimePattern)))
							continue;
						
						var date = self.trim((!self.isUndefined(matched[2])? matched[2]: ""));
						var time = self.trim((!self.isUndefined(matched[3])? matched[3]: ""));
						if(date == "" && time.indexOf("/") >= 0)
						{
							date = time;
							time = "";
						}
						
						if(matched = date.match(self.datePattern))
						{
							// Date
							year = (!self.isUndefined(matched[2])? matched[2]: "");
							month = (!self.isUndefined(matched[3])? matched[3]: "");
							day = (!self.isUndefined(matched[4])? matched[4]: "");
						}
						
						if(matched = time.match(self.timePattern))
						{
							// Time
							hour = (!self.isUndefined(matched[1])? matched[1]: "");
							minute = (!self.isUndefined(matched[3])? matched[3]: "");
						}
						
						span.push({"year":year, "month":month, "day":day, "hour":hour, "minute":minute});
					}
					
					condition.spans.push(span);
				}
				
				if(weekdays != null)
				{
					for(var i=0; i<weekdays.length; i++)
					{
						var weekday = self.trim(weekdays[i]).toLowerCase();
						if(config.available_weekdays[weekday])
						{
							condition.weekdays.push(weekday);
						}
					}
				}
				
				if(content != null)
				{
					condition.content = content;
				}
				else
				{
					if(!(condition.content = self.parseElement(true)))
						return false;
				}
				
				children.push(condition);
			}
			else
			{
				return false;
			}
		}
		
		return {"root":!block, "children":children, "else_index":else_index};
	};
	
	
	//// Utility methods
	self.isUndefined = function(obj)
	{
		return (typeof(obj) == "undefined");
	};
	
	
	self.trim = function(text)
	{
		return text.replace(/^\s*(.+?)\s*$/, "$1");
	};
};
